
package com.ecom.quariument.component;

import com.ecom.quariument.ejb.User;
import com.ecom.quariument.event.QuariumEvent;
import com.ecom.quariument.event.QuariumEventBus;
import com.vaadin.data.Item;
//import com.vaadin.data.Property;
//import com.vaadin.data.Property.ValueChangeListener;
//import com.vaadin.data.Validator.InvalidValueException;
import com.vaadin.data.fieldgroup.BeanFieldGroup;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.fieldgroup.PropertyId;
import com.vaadin.data.util.BeanItem;
//import com.vaadin.data.validator.BeanValidator;
import com.vaadin.data.validator.EmailValidator;
import com.vaadin.data.validator.IntegerRangeValidator;
import com.vaadin.data.validator.NullValidator;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.event.ShortcutAction;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.Page;
import com.vaadin.server.Responsive;
import com.vaadin.server.Sizeable;
import com.vaadin.server.ThemeResource;
//import com.vaadin.server.UserError;
import com.vaadin.shared.Position;
import com.vaadin.shared.ui.MarginInfo;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
//import com.vaadin.ui.CheckBox;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Image;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.TabSheet;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.themes.ValoTheme;
import java.io.Serializable;
import java.lang.reflect.Method;

/**
 *
 * @author David
 */
@SuppressWarnings("serial")
public class UserEditor extends Window implements Button.ClickListener {

    public static final String ID = "usereditorswindow";
    //MUST Implement
    private final int MAXDIRSIZE= 2000;
    private final int MAXMEMSIZE = 40000;

    private final BeanFieldGroup<User> fieldGroup;
    private Button ok;
    private Button cancel;
    private BeanItem<User> user;
    
    @PropertyId("firstName")
    private TextField firstNameField;
    @PropertyId("lastName")
    private TextField lastNameField;
    @PropertyId("title")
    private ComboBox titleField;
    @PropertyId("role")
    private ComboBox roleField;
    
    @PropertyId("male")
    private OptionGroup sexField;
    @PropertyId("email")
    private TextField emailField;
    @PropertyId("location")
    private TextField locationField;
    @PropertyId("phone")
    private TextField phoneField;
    @PropertyId("newsletterSubscription")
    private OptionalSelect<Integer> newsletterField;
    @PropertyId("website")
    private TextField websiteField;
    @PropertyId("bio")
    private TextArea bioField;
    @PropertyId("directory")
    private TextField directoryField;
    @PropertyId("memorySize")
    private TextField memorySizeField;
    @PropertyId("directorySize")
    private TextField directorySizeField;
    @PropertyId("dedicatedProcessor")
    private OptionalSelect<Integer> dedicatedProcessorField;
    
    private OptionalSelect<Integer> userPackage;
    

    

    public UserEditor(final Item user) {
        addStyleName("profile-window");
        setId(ID);
        Responsive.makeResponsive(this);

        setModal(true);
        setCloseShortcut(ShortcutAction.KeyCode.ESCAPE, null);
        setResizable(false);
        setClosable(false);
        setHeight(90.0f, Sizeable.Unit.PERCENTAGE);
        //setWidth(50.0f, Sizeable.Unit.PERCENTAGE);
        
        this.user = (BeanItem<User>) user;
        VerticalLayout content = new VerticalLayout();
        content.setSizeFull();
        content.setMargin(new MarginInfo(true, false, false, false));
        setContent(content);

        TabSheet detailsWrapper = new TabSheet();
        detailsWrapper.setSizeFull();
        detailsWrapper.addStyleName(ValoTheme.TABSHEET_PADDED_TABBAR);
        detailsWrapper.addStyleName(ValoTheme.TABSHEET_ICONS_ON_TOP);
        detailsWrapper.addStyleName(ValoTheme.TABSHEET_CENTERED_TABS);
        content.addComponent(detailsWrapper);
        content.setExpandRatio(detailsWrapper, 1f);

        detailsWrapper.addComponent(buildProfileTab());
        detailsWrapper.addComponent(buildPreferencesTab());

        content.addComponent(buildFooter());

        /* 
        *  BeanFieldGroup binding for Bean Validation
        */
        fieldGroup = new BeanFieldGroup<>(User.class);
        fieldGroup.setBuffered(true);
        fieldGroup.bindMemberFields(this);
        fieldGroup.setItemDataSource(user);
    }

    private Component buildPreferencesTab() {
        VerticalLayout root = new VerticalLayout();
        root.setCaption("Preferences");
        root.setIcon(FontAwesome.COGS);
        root.setSpacing(true);
        root.setMargin(true);
        root.setSizeFull();
        
        FormLayout details = new FormLayout();
        details.addStyleName(ValoTheme.FORMLAYOUT_LIGHT);
        root.addComponent(details);
        root.setExpandRatio(details, 1);
        
        /* with addvalidator without annotations in User.calss
        *  it works. With Bean Validation it fails it focus is lost and
        * content is not valid
        */
        
        roleField = new ComboBox("Role");
        roleField.setInputPrompt("Please specify");
        roleField.addItem("admin");
        roleField.addItem("user");
        //roleField.addValidator(new NullValidator("Choose role",false));
        details.addComponent(roleField);
        
        directoryField = new TextField("Directory Path");
        directoryField.setImmediate(true);
        //TO DO Make my validator here!!!
        details.addComponent(directoryField);

        directorySizeField = new TextField("Directory Size");
        directorySizeField.setImmediate(true);
        //directorySizeField.addValidator(new IntegerRangeValidator("Size must be between 10 and "+MAXDIRSIZE,10,MAXDIRSIZE));
        details.addComponent(directorySizeField);
        
        memorySizeField = new TextField("Memory Size");
        memorySizeField.setImmediate(true);
        //memorySizeField.addValidator(new IntegerRangeValidator("Size must be between 1000 and "+MAXMEMSIZE,1000,MAXMEMSIZE));
        
        details.addComponent(memorySizeField);
        
        dedicatedProcessorField = new OptionalSelect<Integer>("Dedicated Processors");
        for (int i=1;i<9;i++){
            dedicatedProcessorField.addOption(i,Integer.toString(i));
        }
        details.addComponent(dedicatedProcessorField);
                        
        details.addComponent(dedicatedProcessorField);

        return root;
    }

    private Component buildProfileTab() {
        HorizontalLayout root = new HorizontalLayout();
        root.setCaption("Profile");
        root.setIcon(FontAwesome.USER);
        root.setWidth(100.0f, Sizeable.Unit.PERCENTAGE);
        root.setSpacing(true);
        root.setMargin(true);
        root.addStyleName("profile-form");

        VerticalLayout pic = new VerticalLayout();
        pic.setSizeUndefined();
        pic.setSpacing(true);
        Image profilePic = new Image(null, new ThemeResource(
                "img/profile-pic-300px.jpg"));
        profilePic.setWidth(100.0f, Sizeable.Unit.PIXELS);
        pic.addComponent(profilePic);

        Button upload = new Button("Change", new Button.ClickListener() {
            @Override
            public void buttonClick(Button.ClickEvent event) {
                Notification.show("Not neede maybe");
            }
        });
        upload.addStyleName(ValoTheme.BUTTON_TINY);
        pic.addComponent(upload);

        root.addComponent(pic);

        FormLayout details = new FormLayout();
        details.addStyleName(ValoTheme.FORMLAYOUT_LIGHT);
        root.addComponent(details);
        root.setExpandRatio(details, 1);

        firstNameField = new TextField("First Name");
        firstNameField.setImmediate(true);
        //firstNameField.addValidator(new StringLengthValidator(
        //        "Name must be between 2-32 (was {0})",2, 32, true));
        details.addComponent(firstNameField);
        lastNameField = new TextField("Last Name");
        lastNameField.setImmediate(true);
        //lastNameField.addValidator(new StringLengthValidator(
        //        "Name must be between 2-32 (was {0})",2, 32, true));
        details.addComponent(lastNameField);

        titleField = new ComboBox("Title");
        titleField.setInputPrompt("Please specify");
        titleField.addItem("Mr.");
        titleField.addItem("Mrs.");
        titleField.addItem("Ms.");
        titleField.setNewItemsAllowed(true);
        details.addComponent(titleField);

        sexField = new OptionGroup("Sex");
        sexField.addItem(Boolean.FALSE);
        sexField.setItemCaption(Boolean.FALSE, "Female");
        sexField.addItem(Boolean.TRUE);
        sexField.setItemCaption(Boolean.TRUE, "Male");
        sexField.addStyleName("horizontal");
        details.addComponent(sexField);

        Label section = new Label("Contact Info");
        section.addStyleName(ValoTheme.LABEL_H4);
        section.addStyleName(ValoTheme.LABEL_COLORED);
        details.addComponent(section);

        emailField = new TextField("Email");
        emailField.setWidth("100%");
        //emailField.addValidator(new EmailValidator("Not proper email format"));
        details.addComponent(emailField);

        locationField = new TextField("Location");
        locationField.setWidth("100%");
        locationField.setNullRepresentation("");
        details.addComponent(locationField);

        phoneField = new TextField("Phone");
        phoneField.setWidth("100%");
        phoneField.setNullRepresentation("");
        details.addComponent(phoneField);

        newsletterField = new OptionalSelect<Integer>("Subscribe to Newsletter");
        newsletterField.addOption(0, "Daily");
        newsletterField.addOption(1, "Weekly");
        newsletterField.addOption(2, "Monthly");
        details.addComponent(newsletterField);

        section = new Label("Additional Info");
        section.addStyleName(ValoTheme.LABEL_H4);
        section.addStyleName(ValoTheme.LABEL_COLORED);
        details.addComponent(section);

        websiteField = new TextField("Website");
        websiteField.setInputPrompt("http://");
        websiteField.setWidth("100%");
        websiteField.setNullRepresentation("");
        details.addComponent(websiteField);

        bioField = new TextArea("Bio");
        bioField.setWidth("100%");
        bioField.setRows(4);
        bioField.setNullRepresentation("");
        details.addComponent(bioField);

        return root;
    }

    private Component buildFooter() {
        HorizontalLayout footer = new HorizontalLayout();
        footer.addStyleName(ValoTheme.WINDOW_BOTTOM_TOOLBAR);
        footer.setWidth(100.0f, Sizeable.Unit.PERCENTAGE);

        ok = new Button("OK");
        ok.addStyleName(ValoTheme.BUTTON_PRIMARY);
        ok.addClickListener(this);
        ok.focus();
        cancel = new Button("Cancel");
        cancel.addClickListener(this);
        footer.addComponent(ok);
        footer.addComponent(cancel);
        footer.setComponentAlignment(ok, Alignment.TOP_RIGHT);
        return footer;
    }

    public static void open(final Item user, final boolean preferencesTabActive) {
        QuariumEventBus.post(new QuariumEvent.CloseOpenWindowsEvent());
        Window w = new UserEditor(user);
        UI.getCurrent().addWindow(w);
        w.focus();
    }
    
    @Override
    public void buttonClick(Button.ClickEvent event) {
        if (event.getButton()== ok){
                try {
                    fieldGroup.commit();
                    

                    Notification success = new Notification(
                            "Profile updated successfully");
                    success.setDelayMsec(1000);
                    success.setStyleName("bar success small");
                    success.setPosition(Position.BOTTOM_CENTER);
                    success.show(Page.getCurrent());
                    fireEvent(new EditorSavedEvent(this));

                   // QuariumEventBus.post(new QuariumEvent.ProfileUpdatedEvent());
                    close();
                } catch (FieldGroup.CommitException e) {
                    Notification.show("Error by commiting form:"+e.getMessage()+e.getFieldGroup(),
                            Notification.Type.ERROR_MESSAGE);
                }
        } else if (event.getButton() == cancel) {
            fieldGroup.discard();
            close();
        }
            }

    public static class EditorSavedEvent extends Component.Event {
      
      public EditorSavedEvent(Component source) {
          super(source);
      }
    }
    
    public interface EditorSavedListener extends Serializable {
        public void editorSaved(EditorSavedEvent event);
    }
    
    public void removeListener(EditorSavedListener listener) {
        removeListener(EditorSavedEvent.class, listener);
    }
    
    public void addListener(EditorSavedListener listener) {
        try {
            Method method = EditorSavedListener.class.getDeclaredMethod(
                    "editorSaved", new Class[] { EditorSavedEvent.class });
            addListener(EditorSavedEvent.class, listener, method);
        } catch (final java.lang.NoSuchMethodException e) {
            throw new java.lang.RuntimeException(
                    "Internal error, editor saved method not found");
        }
    }

}



/*
public class UserEditor extends Window implements Button.ClickListener {
    public static final String ID = "useradminwindow";

    private final FieldGroup fieldGroup;
    private Button ok;
    private final Item userItem;
    
     * Fields for editing the User object are defined here as class members.
     * They are later bound to a FieldGroup by calling
     * fieldGroup.bindMemberFields(this). The Fields' values don't need to be
     * explicitly set, calling fieldGroup.setItemDataSource(user) synchronizes
     * the fields with the user object.
     
    @PropertyId("firstName")
    private TextField firstNameField;
    @PropertyId("lastName")
    private TextField lastNameField;
    @PropertyId("title")
    private ComboBox titleField;
    @PropertyId("male")
    private OptionGroup sexField;
    @PropertyId("email")
    private TextField emailField;
    @PropertyId("location")
    private TextField locationField;
    @PropertyId("phone")
    private TextField phoneField;
    @PropertyId("newsletterSubscription")
    private OptionalSelect<Integer> newsletterField;
    @PropertyId("website")
    private TextField websiteField;
    @PropertyId("bio")
    private TextArea bioField;
    

    public UserEditor(Item userItem) {
        this.userItem = userItem;
        addStyleName("profile-window");
        setId(ID);
        Responsive.makeResponsive(this);

        setModal(true);
        setCloseShortcut(ShortcutAction.KeyCode.ESCAPE, null);
        setResizable(false);
        setClosable(false);
        setHeight(90.0f, Sizeable.Unit.PERCENTAGE);
        //setWidth(50.0f, Sizeable.Unit.PERCENTAGE);

        VerticalLayout content = new VerticalLayout();
        content.setSizeFull();
        content.setMargin(new MarginInfo(true, false, false, false));
        setContent(content);

        TabSheet detailsWrapper = new TabSheet();
        detailsWrapper.setSizeFull();
        detailsWrapper.addStyleName(ValoTheme.TABSHEET_PADDED_TABBAR);
        detailsWrapper.addStyleName(ValoTheme.TABSHEET_ICONS_ON_TOP);
        detailsWrapper.addStyleName(ValoTheme.TABSHEET_CENTERED_TABS);
        content.addComponent(detailsWrapper);
        content.setExpandRatio(detailsWrapper, 1f);
        
        Component profile = buildProfileTab();

        detailsWrapper.addComponent(profile);
        detailsWrapper.addComponent(buildPreferencesTab());

        content.addComponent(buildFooter());

        fieldGroup = new FieldGroup(userItem);
        fieldGroup.bindMemberFields(this);
        fieldGroup.setItemDataSource(userItem);
    }

    private Component buildPreferencesTab() {
        VerticalLayout root = new VerticalLayout();
        root.setCaption("Preferences");
        root.setIcon(FontAwesome.COGS);
        root.setSpacing(true);
        root.setMargin(true);
        root.setSizeFull();

        Label message = new Label("Not implemented in this demo");
        message.setSizeUndefined();
        message.addStyleName(ValoTheme.LABEL_LIGHT);
        root.addComponent(message);
        root.setComponentAlignment(message, Alignment.MIDDLE_CENTER);

        return root;
    }

    private Component buildProfileTab() {
        HorizontalLayout root = new HorizontalLayout();
        root.setCaption("New User");
        root.setIcon(FontAwesome.USER);
        root.setWidth(100.0f, Sizeable.Unit.PERCENTAGE);
        root.setSpacing(true);
        root.setMargin(true);
        root.addStyleName("profile-form");

        VerticalLayout pic = new VerticalLayout();
        pic.setSizeUndefined();
        pic.setSpacing(true);
        Image profilePic = new Image(null, new ThemeResource(
                "img/profile-pic-300px.jpg"));
        profilePic.setWidth(100.0f, Sizeable.Unit.PIXELS);
        pic.addComponent(profilePic);

        Button upload = new Button("Changeâ€¦", new Button.ClickListener() {
            @Override
            public void buttonClick(Button.ClickEvent event) {
                Notification.show("Not implemented in this demo");
            }
        });
        upload.addStyleName(ValoTheme.BUTTON_TINY);
        pic.addComponent(upload);

        root.addComponent(pic);

        FormLayout details = new FormLayout();
        details.addStyleName(ValoTheme.FORMLAYOUT_LIGHT);
        root.addComponent(details);
        root.setExpandRatio(details, 1);

        firstNameField = new TextField("First Name");
        details.addComponent(firstNameField);
        lastNameField = new TextField("Last Name");
        details.addComponent(lastNameField);

        titleField = new ComboBox("Title");
        titleField.setInputPrompt("Please specify");
        titleField.addItem("Mr.");
        titleField.addItem("Mrs.");
        titleField.addItem("Ms.");
        titleField.setNewItemsAllowed(true);
        details.addComponent(titleField);

        sexField = new OptionGroup("Sex");
        sexField.addItem(Boolean.FALSE);
        sexField.setItemCaption(Boolean.FALSE, "Female");
        sexField.addItem(Boolean.TRUE);
        sexField.setItemCaption(Boolean.TRUE, "Male");
        sexField.addStyleName("horizontal");
        details.addComponent(sexField);

        Label section = new Label("Contact Info");
        section.addStyleName(ValoTheme.LABEL_H4);
        section.addStyleName(ValoTheme.LABEL_COLORED);
        details.addComponent(section);

        emailField = new TextField("Email");
        emailField.setWidth("100%");
        emailField.setRequired(true);
        emailField.setNullRepresentation("");
        details.addComponent(emailField);

        locationField = new TextField("Location");
        locationField.setWidth("100%");
        locationField.setNullRepresentation("");
        locationField.setComponentError(new UserError(
                "This address doesn't exist"));
        details.addComponent(locationField);

        phoneField = new TextField("Phone");
        phoneField.setWidth("100%");
        phoneField.setNullRepresentation("");
        details.addComponent(phoneField);

        newsletterField = new OptionalSelect<Integer>();
        newsletterField.addOption(0, "Daily");
        newsletterField.addOption(1, "Weekly");
        newsletterField.addOption(2, "Monthly");
        details.addComponent(newsletterField);

        section = new Label("Additional Info");
        section.addStyleName(ValoTheme.LABEL_H4);
        section.addStyleName(ValoTheme.LABEL_COLORED);
        details.addComponent(section);

        websiteField = new TextField("Website");
        websiteField.setInputPrompt("http://");
        websiteField.setWidth("100%");
        websiteField.setNullRepresentation("");
        details.addComponent(websiteField);

        bioField = new TextArea("Bio");
        bioField.setWidth("100%");
        bioField.setRows(4);
        bioField.setNullRepresentation("");
        details.addComponent(bioField);

        return root;
    }

    private Component buildFooter() {
        HorizontalLayout footer = new HorizontalLayout();
        footer.addStyleName(ValoTheme.WINDOW_BOTTOM_TOOLBAR);
        footer.setWidth(100.0f, Sizeable.Unit.PERCENTAGE);

        ok = new Button("Save",this);
        ok.addStyleName(ValoTheme.BUTTON_PRIMARY);
        ok.focus();
        footer.addComponent(ok);
        footer.setComponentAlignment(ok, Alignment.TOP_RIGHT);
        return footer;
    }
    
    @Override
    public void buttonClick(Button.ClickEvent event) {
                try {
                    fieldGroup.commit();
                    fireEvent(new EditorSavedEvent(this, userItem));
                    // Updated user should also be persisted to database. But
                    // not in this demo.

                    Notification success = new Notification(
                            "Profile updated successfully");
                    success.setDelayMsec(2000);
                    success.setStyleName("bar success small");
                    success.setPosition(Position.BOTTOM_CENTER);
                    success.show(Page.getCurrent());

                    QuariumEventBus.post(new QuariumEvent.ProfileUpdatedEvent());
                    close();
                } catch (FieldGroup.CommitException e) {
                    Notification.show("Error while updating profile",
                            Notification.Type.ERROR_MESSAGE);
                }

            }
    
   
    public static class EditorSavedEvent extends Component.Event {
      private Item savedItem;
      
      public EditorSavedEvent(Component source, Item savedItem) {
          super(source);
          this.savedItem = savedItem;
      }
      
      public Item getSavedItem() {
          return savedItem;
      }
    }
    
    public interface EditorSavedListener extends Serializable {
        public void editorSaved(EditorSavedEvent event);
    }
    
    public void removeListener(EditorSavedListener listener) {
        removeListener(EditorSavedEvent.class, listener);
    }
    
    public void addListener(EditorSavedListener listener) {
        try {
            Method method = EditorSavedListener.class.getDeclaredMethod(
                    "editorSaved", new Class[] { EditorSavedEvent.class });
            addListener(EditorSavedEvent.class, listener, method);
        } catch (final java.lang.NoSuchMethodException e) {
            throw new java.lang.RuntimeException(
                    "Internal error, editor saved method not found");
        }
    }

    public static void open(Item userItem) {
        QuariumEventBus.post(new QuariumEvent.CloseOpenWindowsEvent());
        Window w = new UserEditor(userItem);
        UI.getCurrent().addWindow(w);
        w.focus();
    }
    
}*/
