package com.ecom.quariument.ui;



import java.util.Locale;

import com.google.common.eventbus.Subscribe;
import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Title;
import com.vaadin.annotations.Widgetset;
import com.ecom.quariument.ejb.DataProvider;
import com.ecom.quariument.ejb.QuariumDataProvider;
import com.ecom.quariument.ejb.User;
import com.ecom.quariument.event.QuariumEvent.BrowserResizeEvent;
import com.ecom.quariument.event.QuariumEvent.CloseOpenWindowsEvent;
import com.ecom.quariument.event.QuariumEvent.UserLoggedOutEvent;
import com.ecom.quariument.event.QuariumEvent.UserLoginRequestedEvent;
import com.ecom.quariument.event.QuariumEventBus;
import com.ecom.quariument.view.LoginView;
import com.ecom.quariument.view.MainView;

import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.server.Page;
import com.vaadin.server.Page.BrowserWindowResizeEvent;
import com.vaadin.server.Page.BrowserWindowResizeListener;
import com.vaadin.server.Responsive;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.UI;
import static com.vaadin.ui.UI.getCurrent;
import com.vaadin.ui.Window;
import com.vaadin.ui.themes.ValoTheme;
import javax.servlet.annotation.WebServlet;

import com.vaadin.annotations.Push;

//@Theme("dashboard")
@Theme("dashboard")
//@Widgetset("com.vaadin.demo.dashboard.DashboardWidgetSet")
@Title("Quarium")
@SuppressWarnings("serial")
@Push

public final class MyUI extends UI {
    
    

    @WebServlet(value = "/*", name="MyUIServlet", asyncSupported = true)
    @VaadinServletConfiguration(productionMode = true, ui = MyUI.class, widgetset = "com.ecom.quariument.ui.AppWidgetSet")
    public static class MyUIServlet extends VaadinServlet {
    }
    /*
     * This field stores an access to the dummy backend layer. In real
     * applications you most likely gain access to your beans trough lookup or
     * injection; and not in the UI but somewhere closer to where they're
     * actually accessed.
     */
    private final DataProvider dataProvider = new QuariumDataProvider();
    private final QuariumEventBus quariumEventbus = new QuariumEventBus();

    
    
    
    @Override
    protected void init(final VaadinRequest request) {
        setLocale(Locale.US);

        QuariumEventBus.register(this);
        Responsive.makeResponsive(this);
        addStyleName(ValoTheme.UI_WITH_MENU);

        updateContent();

        // Some views need to be aware of browser resize events so a
        // BrowserResizeEvent gets fired to the event bus on every occasion.
        Page.getCurrent().addBrowserWindowResizeListener(
                new BrowserWindowResizeListener() {
                    @Override
                    public void browserWindowResized(
                            final BrowserWindowResizeEvent event) {
                        QuariumEventBus.post(new BrowserResizeEvent());
                    }
                });
    }

    /**
     * Updates the correct content for this UI based on the current user status.
     * If the user is logged in with appropriate privileges, main view is shown.
     * Otherwise login view is shown.
     */
    private void updateContent() {
        User user = (User) VaadinSession.getCurrent().getAttribute(
                User.class.getName());
        if (user != null ) {
            // Authenticated user
            setContent(new MainView());
            removeStyleName("loginview");
            getNavigator().navigateTo(getNavigator().getState());
        } else {
            setContent(new LoginView());
            addStyleName("loginview");
        }
    }

    @Subscribe
    public void userLoginRequested(final UserLoginRequestedEvent event) {
        User user = getDataProvider().authenticate(event.getUserName(),
                event.getPassword());
        VaadinSession.getCurrent().setAttribute(User.class.getName(), user);
        updateContent();
    }

    @Subscribe
    public void userLoggedOut(final UserLoggedOutEvent event) {
        // When the user logs out, current VaadinSession gets closed and the
        // page gets reloaded on the login screen. Do notice the this doesn't
        // invalidate the current HttpSession.
        VaadinSession.getCurrent().close();
        Page.getCurrent().reload();
    }

    @Subscribe
    public void closeOpenWindows(final CloseOpenWindowsEvent event) {
        for (Window window : getWindows()) {
            window.close();
        }
    }

    /**
     * @return An instance for accessing the (dummy) services layer.
     */
    public static DataProvider getDataProvider() {
        return ((MyUI) getCurrent()).dataProvider;
    }

    public static QuariumEventBus getDashboardEventbus() {
        return ((MyUI) getCurrent()).quariumEventbus;
    }
}

/*import javax.servlet.annotation.WebServlet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map.Entry;

import javax.servlet.ServletException;

import com.vaadin.annotations.PreserveOnRefresh;
import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Title;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.data.Container;
import com.vaadin.data.Container.Hierarchical;
import com.vaadin.data.Item;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.data.util.HierarchicalContainer;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.event.Action;
import com.vaadin.event.Action.Handler;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.Page;
import com.vaadin.server.Resource;
import com.vaadin.server.Responsive;
import com.vaadin.server.ThemeResource;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.shared.ui.label.ContentMode;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Component;
import com.vaadin.ui.ComponentContainer;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.MenuBar;
import com.vaadin.ui.MenuBar.MenuItem;
import com.vaadin.ui.NativeSelect;
import com.vaadin.ui.Notification;
import com.vaadin.ui.UI;
import com.vaadin.ui.themes.ValoTheme;
import com.vaadin.annotations.Push;

/* Database dependencies /
import com.ecom.quarium.database.*;

@Theme("mytheme")
@SuppressWarnings("serial")
@Push
public class InfiniteUI extends UI
{

    @WebServlet(value = "/*", asyncSupported = true)
    @VaadinServletConfiguration(productionMode = false, ui = InfiniteUI.class, widgetset = "com.ecom.quarium.ui.AppWidgetSet")
    public static class Servlet extends VaadinServlet {
    }
        InfiniteLayout root = new InfiniteLayout();
    ComponentContainer viewDisplay = root.getContentContainer();
    CssLayout menu = new CssLayout();
    CssLayout menuItemsLayout = new CssLayout();
    {
        menu.setId("testMenu");
    }
    
    private Navigator navigator;
    private final LinkedHashMap<String, String> menuItems = new LinkedHashMap<String, String>();
    
    @Override
    protected void init(VaadinRequest request) {
        
        /*DatabaseProviderFactory factory = new DatabaseProviderFactory();
        IDatabaseProvider provider = factory.createProvider();
        provider.connect(); /
        
         Responsive.makeResponsive(this);
         getPage().setTitle("Quarium");
         setContent(root);
         root.setWidth("100%");
         
         root.addMenu(buildMenu());

        navigator = new Navigator(this, viewDisplay);
        //ide kell egy automata menu betöltő MenuClassból olvasva
        navigator.addView("common", CommonParts.class);
        navigator.addView("labels", Labels.class);
        navigator.addView("dashboard", Dashboard.class);
        navigator.addView("myapplications", MyApplications.class);
        navigator.addView("test", Test.class);
        
        //alapban kihez navigál
        final String f = Page.getCurrent().getUriFragment();
        if (f == null || f.equals("")) {
            navigator.navigateTo("common");
        }
        //hiba esetén kihez navigál
        navigator.setErrorView(CommonParts.class);
        //nézet ablak váltáskor mi történjen
        navigator.addViewChangeListener(new ViewChangeListener() {
            //alapban semmi esetleg jogosultság ellenőrzés
            @Override
            public boolean beforeViewChange(final ViewChangeEvent event) {
                return true;
            }
            //váltás után ?
            @Override
            public void afterViewChange(final ViewChangeEvent event) {
                for (final Iterator<Component> it = menuItemsLayout.iterator(); it
                        .hasNext();) {
                    it.next().removeStyleName("selected");
                }
                for (final Entry<String, String> item : menuItems.entrySet()) {
                    if (event.getViewName().equals(item.getKey())) {
                        for (final Iterator<Component> it = menuItemsLayout
                                .iterator(); it.hasNext();) {
                            final Component c = it.next();
                            if (c.getCaption() != null
                                    && c.getCaption().startsWith(
                                            item.getValue())) {
                                c.addStyleName("selected");
                                break;
                            }
                        }
                        break;
                    }
                }
                menu.removeStyleName("valo-menu-visible");
            }
        });    
    }
    
    CssLayout buildMenu() {
        // Ide kell egy betöltő ami a HashMap-hez hozzáadja automatikusan talán feljebb kéne ezt a navigátorral közösen
            
        menuItems.put("common", "Common UI Elements");
        menuItems.put("labels", "Labels");
        menuItems.put("dashboard","Dashboard");
        menuItems.put("myapplications","Applications");
        menuItems.put("test","Test");
        
        menuItemsLayout.setPrimaryStyleName("valo-menuitems");
        menu.addComponent(menuItemsLayout);
        for (final Entry<String, String> item : menuItems.entrySet()) {
                final Button b = new Button(item.getValue(), new ClickListener() {
                @Override
                public void buttonClick(final ClickEvent event) {
                    navigator.navigateTo(item.getKey());
                }
            });
                b.setHtmlContentAllowed(true);
                b.setPrimaryStyleName("valo-menu-item");
                //icon kép b.setIcon(valami)
                menuItemsLayout.addComponent(b);
        }
        return menu;
    }
    
}

*/