/*
 * Copyright (C) 2015 David
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.ecom.quariument.ui;

/**
 *
 * @author David
 */

import com.ecom.quariument.ejb.User;
import com.ecom.quariument.event.QuariumEvent.BrowserResizeEvent;
import com.ecom.quariument.event.QuariumEvent.CloseOpenWindowsEvent;
import com.ecom.quariument.event.QuariumEvent.PostViewChangeEvent;
import com.ecom.quariument.event.QuariumEventBus;
import com.ecom.quariument.view.QuariumViewType;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.navigator.ViewProvider;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.ComponentContainer;
import com.vaadin.ui.UI;

@SuppressWarnings("serial")
public class QuariumNavigator extends Navigator {


    private static final QuariumViewType ERROR_VIEW = QuariumViewType.INITIAL;
    private ViewProvider errorViewProvider;

    public QuariumNavigator(final ComponentContainer container) {
        super(UI.getCurrent(), container);

        String host = getUI().getPage().getLocation().getHost();
        initViewChangeListener();
        initViewProviders();

    }

    private void initViewChangeListener() {
        addViewChangeListener(new ViewChangeListener() {

            @Override
            public boolean beforeViewChange(final ViewChangeListener.ViewChangeEvent event) {
                // Since there's no conditions in switching between the views
                // we can always return true.
                return true;
            }

            @Override
            public void afterViewChange(final ViewChangeListener.ViewChangeEvent event) {
                QuariumViewType view = QuariumViewType.getByViewName(event
                        .getViewName());
                // Appropriate events get fired after the view is changed.
                QuariumEventBus.post(new PostViewChangeEvent(view));
                QuariumEventBus.post(new BrowserResizeEvent());
                QuariumEventBus.post(new CloseOpenWindowsEvent());

            }
        });
    }

    private void initViewProviders() {
        // A dedicated view provider is added for each separate view type
        User user = (User) VaadinSession.getCurrent().getAttribute(
                User.class.getName());
        for (final QuariumViewType viewType : QuariumViewType.values()) {
            if ("admin".equals(user.getRole()) || viewType.isUserSees()) {
            ViewProvider viewProvider = new Navigator.ClassBasedViewProvider(
                    viewType.getViewName(), viewType.getViewClass()) {

                // This field caches an already initialized view instance if the
                // view should be cached (stateful views).
                private View cachedInstance;

                @Override
                public View getView(final String viewName) {
                    View result = null;
                    if (viewType.getViewName().equals(viewName)) {
                        if (viewType.isStateful()) {
                            // Stateful views get lazily instantiated
                            if (cachedInstance == null) {
                                cachedInstance = super.getView(viewType
                                        .getViewName());
                            }
                            result = cachedInstance;
                        } else {
                            // Non-stateful views get instantiated every time
                            // they're navigated to
                            result = super.getView(viewType.getViewName());
                        }
                    }
                    return result;
                }
            };

            if (viewType == ERROR_VIEW) {
                errorViewProvider = viewProvider;
            }

            addProvider(viewProvider);
        }
        }

        setErrorProvider(new ViewProvider() {
            @Override
            public String getViewName(final String viewAndParameters) {
                return ERROR_VIEW.getViewName();
            }

            @Override
            public View getView(final String viewName) {
                return errorViewProvider.getView(ERROR_VIEW.getViewName());
            }
        });
    }
}
