/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ecom.quariument.view.admin;

import com.ecom.quariument.component.UserEditor;
import com.ecom.quariument.component.UserEditor.EditorSavedEvent;
import com.ecom.quariument.component.UserEditor.EditorSavedListener;
import com.ecom.quariument.ejb.User;
import com.ecom.quariument.event.QuariumEvent;
import com.ecom.quariument.event.QuariumEventBus;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.event.ItemClickEvent;
import com.vaadin.event.ItemClickEvent.ItemClickListener;
import com.vaadin.event.LayoutEvents;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.Responsive;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Component;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.themes.ValoTheme;
import java.util.Iterator;

/**
 *
 * @author David
 */
public class AdminView extends Panel implements View{
    private final VerticalLayout root;
    private VerticalLayout dashboardPanels;
    Button addUser = new Button("Add");
    Button removeUser = new Button("Remove");
    Button editUser = new Button("Edit");
    BeanItemContainer<User> container = new BeanItemContainer<User>(User.class);
    Table userTable = new Table("Users",container);
    
    public AdminView(){
        addStyleName(ValoTheme.PANEL_BORDERLESS);
        setSizeFull();
        QuariumEventBus.register(this);
        
        root = new VerticalLayout();
        root.setSizeFull();
        root.setMargin(true);
        root.addStyleName("dashboard-view");
        setContent(root);
        Responsive.makeResponsive(root);
        Component content = buildContent();
        root.addComponent(content);
        root.setExpandRatio(content, 1);
        root.addComponent(content);
            
        // All the open sub-windows should be closed whenever the root layout
        // gets clicked.
        root.addLayoutClickListener(new LayoutEvents.LayoutClickListener() {
            @Override
            public void layoutClick(final LayoutEvents.LayoutClickEvent event) {
                QuariumEventBus.post(new QuariumEvent.CloseOpenWindowsEvent());
            }
        });      
    }
    
    private Component buildContent(){
        dashboardPanels = new VerticalLayout();
        dashboardPanels.setSpacing(true);
        dashboardPanels.addStyleName("dashboard-panels");
        Responsive.makeResponsive(dashboardPanels);

        dashboardPanels.addComponent(manageUsers());
        
        return dashboardPanels;
    }
    
    Panel manageUsers(){
        Panel p = new Panel("Manage Users");
        VerticalLayout layout = new VerticalLayout();
        
        userTable.setSelectable(true);
        userTable.setImmediate(true);
        userTable.setSizeFull();
        userTable.addListener(new Property.ValueChangeListener(){
            
            @Override
            public void valueChange(ValueChangeEvent event){
                setModificationsEnabled(event.getProperty().getValue() != null);
            }
            
            private void setModificationsEnabled(boolean b) {
                removeUser.setEnabled(b);
                editUser.setEnabled(b);
            }
        });
        userTable.addListener(new ItemClickListener() {
            @Override
            public void itemClick(ItemClickEvent event) {
               if (event.isDoubleClick()) {
                   userTable.select(event.getItemId());
               } 
            }
        });
        userTable.setColumnCollapsingAllowed(true);
        for(Iterator i=container.getContainerPropertyIds().iterator(); i.hasNext();){
            String id = (String) i.next();
            userTable.setColumnCollapsed(id, true);
        }
        userTable.setColumnCollapsed("firstName", false);
        userTable.setColumnCollapsed("lastName", false);
        
        HorizontalLayout toolbar = new HorizontalLayout();
        addUser.addClickListener(new Button.ClickListener(){
            @Override
            public void buttonClick(ClickEvent event){
                final BeanItem<User> newUserItem = new BeanItem(new User());
                UserEditor userEditor = new UserEditor(newUserItem);
                userEditor.addListener(new EditorSavedListener(){
                    
                    @Override
                    public void editorSaved(EditorSavedEvent event) {
                        container.addBean(newUserItem.getBean());
                        Notification.show("Applied:"+newUserItem.getBean().getFirstName()+
                                container.size());
                        userTable.markAsDirty();
                    }
                });
                UI.getCurrent().addWindow(userEditor);
            }
        });
        removeUser.addClickListener(new Button.ClickListener() {
            @Override
            public void buttonClick(ClickEvent event) {
                container.removeItem(userTable.getValue());
            }
        });
        removeUser.setEnabled(false);
        
        editUser.addClickListener(new Button.ClickListener() {
            @Override
            public void buttonClick(ClickEvent event) {
                UI.getCurrent().addWindow(
                        new UserEditor(container.getItem(userTable.getValue())));
            }
        });
        editUser.setEnabled(false);
        
        toolbar.addComponent(addUser);
        toolbar.addComponent(editUser);
        toolbar.addComponent(removeUser);
        toolbar.setWidth("100%");
        layout.addComponent(toolbar);
        layout.addComponent(userTable);
        layout.setExpandRatio(userTable, 1);
        layout.setSizeFull();
        p.setContent(layout);
        
        return p;
    }
    
    @Override
    public void enter(final ViewChangeListener.ViewChangeEvent event) {
        // TODO Auto-generated method stub

    }
}
