/*
 * Copyright (C) 2015 David
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.ecom.quariument.view.initial;

/**
 *
 * @author David
 */

import com.ecom.quariument.ejb.QuariumNotification;
import com.ecom.quariument.event.QuariumEvent.CloseOpenWindowsEvent;
import com.ecom.quariument.event.QuariumEvent.NotificationsCountUpdatedEvent;
import com.ecom.quariument.event.QuariumEventBus;
import com.ecom.quariument.ejb.ApplicationStatus;
import com.ecom.quariument.ui.MyUI;
import com.ecom.quariument.ejb.ServiceData;
import com.google.common.eventbus.Subscribe;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Table;
import com.vaadin.ui.Notification;
import com.vaadin.event.LayoutEvents;
import com.vaadin.event.ShortcutAction;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.Responsive;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Component;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.themes.ValoTheme;
import java.util.Collection;

/**
 *
 * @author David
 */
public class InitialView extends Panel implements View {
        private final VerticalLayout root;
        private VerticalLayout dashboardPanels;
        private Label titleLabel;
        private NotificationsButton notificationsButton;
        private Window notificationsWindow;
        
        public static final String TITLE_ID = "initialview-title";
        
        private BeanItemContainer<ApplicationStatus> container = new BeanItemContainer<ApplicationStatus>(ApplicationStatus.class);
        private Label test = new Label("Test:");
        private Table table;
        
        public InitialView() {
            addStyleName(ValoTheme.PANEL_BORDERLESS);
            setSizeFull();
            QuariumEventBus.register(this);

            root = new VerticalLayout();
            root.setSizeFull();
            root.setMargin(true);
            root.addStyleName("dashboard-view");
            setContent(root);
            Responsive.makeResponsive(root);
            root.addComponent(buildHeader());
            
            Component content = buildContent();
            root.addComponent(content);
            root.setExpandRatio(content, 1);
            
            new RefressData().start();
            
            // All the open sub-windows should be closed whenever the root layout
        // gets clicked.
        root.addLayoutClickListener(new LayoutEvents.LayoutClickListener() {
            @Override
            public void layoutClick(final LayoutEvents.LayoutClickEvent event) {
                QuariumEventBus.post(new CloseOpenWindowsEvent());
            }
        });
    }
        
        private Component buildHeader() {
        HorizontalLayout header = new HorizontalLayout();
        header.addStyleName("viewheader");
        header.setSpacing(true);

        titleLabel = new Label("Status");
        titleLabel.setId(TITLE_ID);
        titleLabel.setSizeUndefined();
        titleLabel.addStyleName(ValoTheme.LABEL_H1);
        titleLabel.addStyleName(ValoTheme.LABEL_NO_MARGIN);
        header.addComponent(titleLabel);

        notificationsButton = buildNotificationsButton();
        HorizontalLayout tools = new HorizontalLayout(notificationsButton);
        tools.setSpacing(true);
        tools.addStyleName("toolbar");
        tools.setSizeFull();
        header.addComponent(tools);
        header.setComponentAlignment(tools, Alignment.MIDDLE_CENTER);

        return header;
    }
        
    private Component buildContent() {
        dashboardPanels = new VerticalLayout();
        dashboardPanels.setSpacing(true);
        dashboardPanels.addStyleName("dashboard-panels");
        Responsive.makeResponsive(dashboardPanels);

        dashboardPanels.addComponent(serviceStatus());
        dashboardPanels.addComponent(applicationStatus());
        dashboardPanels.addComponent(cassandraTest());
        

        return dashboardPanels;
    }
        
        Panel cassandraTest(){
            Panel p = new Panel("CassandraTest");
            Button btn = new Button("CassandraTest");
            btn.addClickListener(new ClickListener() {
                        @Override
                        public void buttonClick(final ClickEvent event) {
                            Notification.show(MyUI.getDataProvider().getCassandraTest());
                        }
            });
            p.setContent(btn);
            return p;
        }
        

        Panel serviceStatus() {
                Panel p = new Panel("Service status");

                HorizontalLayout pcontent = new HorizontalLayout();
                pcontent.setSpacing(true);
                pcontent.setMargin(true);
                p.setContent(pcontent);
                p.setSizeFull();
                
                // get actual service data
                ServiceData sd;
                sd = MyUI.getDataProvider().getServiceData();
                //adding to layout
                Label dash = new Label("Dash");
                dash.addStyleName(sd.toLabelString(sd.getDash()));
                pcontent.addComponent(dash);
                
                Label glassfish = new Label("Glassfish");
                glassfish.addStyleName(sd.toLabelString(sd.getGlassfish()));
                pcontent.addComponent(glassfish);
                
                Label cassandra = new Label("Cassandra");
                cassandra.addStyleName(sd.toLabelString(sd.getCassandra()));
                pcontent.addComponent(cassandra);
                
                Label proxy = new Label("Proxy");
                proxy.addStyleName(sd.toLabelString(sd.getProxy()));
                pcontent.addComponent(proxy);
                
                return p;
        }
        
        Panel applicationStatus() {
                Panel p = new Panel("Application status");

                VerticalLayout pcontent = new VerticalLayout();
                pcontent.setSpacing(true);
                pcontent.setMargin(true);
                p.setContent(pcontent);
                p.setSizeFull();
                
                container = MyUI.getDataProvider().getApplicationStatusData();
                
                table = new Table();
                table.setContainerDataSource(container);
                table.setPageLength(table.size());
                table.setImmediate(true);
                //table.setEditable(true);
                table.setVisibleColumns( new Object[] {"name", "running","uptime","memory","bandwidth"} );

                
                
                //itt lenne  abetöltő ciklus vége
                pcontent.addComponent(table); 
                pcontent.addComponent(test);
                
                return p;
        }
        
        public class RefressData extends Thread {
            //Table refresh intervaal
            public final int sleepms = 2000; 
            int cont =0;
            @Override
                public void run() {
                    while (true) {
                        
                        try {
                            
                            Thread.sleep(sleepms);
                            getUI().access(new Runnable() {
                            @Override
                            public void run() {
                              container.removeAllItems();
                              BeanItemContainer<ApplicationStatus> repcontainer;
                              repcontainer = MyUI.getDataProvider().getApplicationStatusData();
                              table.setContainerDataSource(repcontainer);
                              table.setVisibleColumns( new Object[] {"name", "running","uptime","memory","bandwidth"} );
                              /*for (Iterator i = repcontainer.getItemIds().iterator(); i.hasNext();) {
                                  // Get the current item identifier, which is an integer.
                                  int iid = (Integer) i.next();
                                  // Now get the actual item from the table.
                                  ApplicationStatus beanItem = repcontainer.getItem(iid).getBean();
                                  container.addBean(beanItem);
                              }*/
                              
                              
                              test.setValue("Tester:"+cont); 
                               cont++;
                            }
                            });
                            
                        } catch (Exception e) {
                           // Notification.show("Warning","Application data table rfresh stopped"
                           //         + e.getMessage(), Notification.Type.ERROR_MESSAGE);
                        }
                    }
                }
                
        }
        
    public static final class NotificationsButton extends Button {
        private static final String STYLE_UNREAD = "unread";
        public static final String ID = "dashboard-notifications";

        public NotificationsButton() {
            setIcon(FontAwesome.BELL);
            setId(ID);
            addStyleName("notifications");
            addStyleName(ValoTheme.BUTTON_ICON_ONLY);
            QuariumEventBus.register(this);
        }

        @Subscribe
        public void updateNotificationsCount(
                final NotificationsCountUpdatedEvent event) {
            setUnreadCount(MyUI.getDataProvider()
                    .getUnreadNotificationsCount());
        }

        public void setUnreadCount(final int count) {
            setCaption(String.valueOf(count));

            String description = "Notifications";
            if (count > 0) {
                addStyleName(STYLE_UNREAD);
                description += " (" + count + " unread)";
            } else {
                removeStyleName(STYLE_UNREAD);
            }
            setDescription(description);
        }
    }
    
    private NotificationsButton buildNotificationsButton() {
        NotificationsButton result = new NotificationsButton();
        result.addClickListener(new Button.ClickListener() {
            @Override
            public void buttonClick(final Button.ClickEvent event) {
                openNotificationsPopup(event);
            }
        });
        return result;
    }
    
    
    
    private void openNotificationsPopup(final Button.ClickEvent event) {
        VerticalLayout notificationsLayout = new VerticalLayout();
        notificationsLayout.setMargin(true);
        notificationsLayout.setSpacing(true);

        Label title = new Label("Notifications");
        title.addStyleName(ValoTheme.LABEL_H3);
        title.addStyleName(ValoTheme.LABEL_NO_MARGIN);
        notificationsLayout.addComponent(title);

        Collection<QuariumNotification> notifications = MyUI
                .getDataProvider().getNotifications();
        QuariumEventBus.post(new NotificationsCountUpdatedEvent());

        for (QuariumNotification notification : notifications) {
            VerticalLayout notificationLayout = new VerticalLayout();
            notificationLayout.addStyleName("notification-item");

            Label titleLabelNotification = new Label(notification.getFirstName() + " "
                    + notification.getLastName() + " "
                    + notification.getAction());
            titleLabelNotification.addStyleName("notification-title");

            Label timeLabel = new Label(notification.getPrettyTime());
            timeLabel.addStyleName("notification-time");

            Label contentLabel = new Label(notification.getContent());
            contentLabel.addStyleName("notification-content");

            notificationLayout.addComponents(titleLabelNotification, timeLabel,
                    contentLabel);
            notificationsLayout.addComponent(notificationLayout);
        }

        HorizontalLayout footer = new HorizontalLayout();
        footer.addStyleName(ValoTheme.WINDOW_BOTTOM_TOOLBAR);
        footer.setWidth("100%");
        Button showAll = new Button("View All Notifications",
                new Button.ClickListener() {
                    @Override
                    public void buttonClick(final Button.ClickEvent event) {
                        Notification.show("Not implemented in this demo");
                    }
                });
        showAll.addStyleName(ValoTheme.BUTTON_BORDERLESS_COLORED);
        showAll.addStyleName(ValoTheme.BUTTON_SMALL);
        footer.addComponent(showAll);
        footer.setComponentAlignment(showAll, Alignment.TOP_CENTER);
        notificationsLayout.addComponent(footer);

        if (notificationsWindow == null) {
            notificationsWindow = new Window();
            notificationsWindow.setWidth(300.0f, Unit.PIXELS);
            notificationsWindow.addStyleName("notifications");
            notificationsWindow.setClosable(false);
            notificationsWindow.setResizable(false);
            notificationsWindow.setDraggable(false);
            notificationsWindow.setCloseShortcut(ShortcutAction.KeyCode.ESCAPE, null);
            notificationsWindow.setContent(notificationsLayout);
        }

        if (!notificationsWindow.isAttached()) {
            notificationsWindow.setPositionY(event.getClientY()
                    - event.getRelativeY() + 40);
            getUI().addWindow(notificationsWindow);
            notificationsWindow.focus();
        } else {
            notificationsWindow.close();
        }
    }

        
        @Override
    public void enter(final ViewChangeEvent event) {
        // TODO Auto-generated method stub

    }
        
}
